@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008-2011 Red Hat Inc.
+ * Copyright (C) 2008-2013 Red Hat Inc.
  *
  * Arnaldo Carvalho de Melo <acme@redhat.com>
  * David Sommerseth <davids@redhat.com>
@@ -22,19 +22,17 @@
 #include <stdint.h>
 #include <unistd.h>
 #include <sys/socket.h>
-#include <net/if.h>
 #include <sys/socket.h>
 #include <sys/ioctl.h>
 #include <sys/types.h>
+#include <ifaddrs.h>
+#include <netlink/route/addr.h>
 
 #include "etherinfo_struct.h"
 #include "etherinfo_obj.h"
 #include "etherinfo.h"
 
-static struct nl_handle *nlconnection = NULL;
-unsigned int nlconnection_users = 0;  /* How many NETLINK users are active? */
-extern PyTypeObject ethtool_etherinfoType;
-extern PyTypeObject ethtool_etherinfoIPv6Type;
+extern PyTypeObject PyEtherInfo_Type;
 
 #ifndef IFF_DYNAMIC
 #define IFF_DYNAMIC     0x8000          /* dialup device with changing addresses*/
@@ -55,55 +53,24 @@ typedef __uint8_t u8;
 static PyObject *get_active_devices(PyObject *self __unused, PyObject *args __unused)
 {
 	PyObject *list;
-	int numreqs = 30;
-	struct ifconf ifc;
-	struct ifreq *ifr;
-	int n;
+	struct ifaddrs *ifaddr, *ifa;
 
-	/* SIOCGIFCONF currently seems to only work properly on AF_INET sockets
-	   (as of 2.1.128) */
-	/* Open control socket. */
-	int skfd = socket(AF_INET, SOCK_DGRAM, 0);
-
-	if (skfd < 0) {
+	if (getifaddrs(&ifaddr) == -1) {
 		PyErr_SetString(PyExc_OSError, strerror(errno));
 		return NULL;
 	}
 
-	ifc.ifc_buf = NULL;
-	for (;;) {
-		ifc.ifc_len = sizeof(struct ifreq) * numreqs;
-		ifc.ifc_buf = realloc(ifc.ifc_buf, ifc.ifc_len);
-
-		if (ioctl(skfd, SIOCGIFCONF, &ifc) < 0) {
-			PyErr_SetString(PyExc_OSError, strerror(errno));
-			free(ifc.ifc_buf);
-			close(skfd);
-			return NULL;
-		}
-
-		if (ifc.ifc_len == (int)sizeof(struct ifreq) * numreqs) {
-			/* assume it overflowed and try again */
-			numreqs += 10;
-			continue;
-		}
-		break;
-	}
-
 	list = PyList_New(0);
-	ifr = ifc.ifc_req;
-	for (n = 0; n < ifc.ifc_len; n += sizeof(struct ifreq)) {
-		if (!(ioctl(skfd, SIOCGIFFLAGS, ifr) < 0))
-			if (ifr->ifr_flags & IFF_UP) {
-				PyObject *str = PyString_FromString(ifr->ifr_name);
-				PyList_Append(list, str);
-				Py_DECREF(str);
-			}
-			ifr++;
+	for (ifa = ifaddr; ifa != NULL; ifa = ifa->ifa_next) {
+		PyObject *str = PyString_FromString(ifa->ifa_name);
+		/* names are not unique (listed for both ipv4 and ipv6) */
+		if (!PySequence_Contains(list, str) && (ifa->ifa_flags & (IFF_UP))) {
+			PyList_Append(list, str);
+		}
+	Py_DECREF(str);
 	}
 
-	free(ifc.ifc_buf);
-	close(skfd);
+	freeifaddrs(ifaddr);
 
 	return list;
 }
@@ -111,7 +78,7 @@ static PyObject *get_active_devices(PyObject *self __unused, PyObject *args __un
 static PyObject *get_devices(PyObject *self __unused, PyObject *args __unused)
 {
 	char buffer[256];
-	char *ret;;
+	char *ret;
 	PyObject *list = PyList_New(0);
 	FILE *fd = fopen(_PATH_PROCNET_DEV, "r");
 
@@ -120,7 +87,13 @@ static PyObject *get_devices(PyObject *self __unused, PyObject *args __unused)
 		return NULL;
 	}
 	/* skip over first two lines */
-	ret = fgets(buffer, 256, fd); ret = fgets(buffer, 256, fd);
+	ret = fgets(buffer, 256, fd);
+	ret = fgets(buffer, 256, fd);
+	if( !ret ) {
+		PyErr_SetString(PyExc_OSError, strerror(errno));
+		return NULL;
+	}
+
 	while (!feof(fd)) {
 		PyObject *str;
 		char *name = buffer;
@@ -129,7 +102,7 @@ static PyObject *get_devices(PyObject *self __unused, PyObject *args __unused)
 		if (fgets(buffer, 256, fd) == NULL)
 			break;
 		/* find colon */
-		while (end && *end != ':')
+		while (*end && *end != ':')
 			end++;
 		*end = 0; /* terminate where colon was */
 		while (*name == ' ')
@@ -147,7 +120,7 @@ static PyObject *get_hwaddress(PyObject *self __unused, PyObject *args)
 {
 	struct ifreq ifr;
 	int fd, err;
-	char *devname;
+	const char *devname;
 	char hwaddr[20];
 
 	if (!PyArg_ParseTuple(args, "s", &devname))
@@ -194,7 +167,7 @@ static PyObject *get_ipaddress(PyObject *self __unused, PyObject *args)
 {
 	struct ifreq ifr;
 	int fd, err;
-	char *devname;
+	const char *devname;
 	char ipaddr[20];
 
 	if (!PyArg_ParseTuple(args, "s", &devname))
@@ -240,14 +213,14 @@ static PyObject *get_ipaddress(PyObject *self __unused, PyObject *args)
  * returned as a list of objects per interface.
  *
  * @param self Not used
- * @param args Python arguments
+ * @param args Python arguments - device name(s) as either a string or a list
  *
  * @return Python list of objects on success, otherwise NULL.
  */
 static PyObject *get_interfaces_info(PyObject *self __unused, PyObject *args) {
-	PyObject *devlist = NULL, *ethinf_py = NULL;
+	PyObject *devlist = NULL;
 	PyObject *inargs = NULL;
-	char **fetch_devs;
+	char **fetch_devs = NULL;
 	int i = 0, fetch_devs_len = 0;
 
 	if (!PyArg_ParseTuple(args, "|O", &inargs)) {
@@ -295,48 +268,27 @@ static PyObject *get_interfaces_info(PyObject *self __unused, PyObject *args) {
 
 	devlist = PyList_New(0);
 	for( i = 0; i < fetch_devs_len; i++ ) {
-		struct etherinfo_obj_data *objdata = NULL;
-
-		/* Allocate memory for data structures for each device */
-		objdata = calloc(1, sizeof(struct etherinfo_obj_data));
-		if( !objdata ) {
-			PyErr_SetString(PyExc_OSError, strerror(errno));
-			return NULL;
-		}
-
-		objdata->ethinfo = calloc(1, sizeof(struct etherinfo));
-		if( !objdata->ethinfo ) {
-			PyErr_SetString(PyExc_OSError, strerror(errno));
-			return NULL;
-		}
+                PyEtherInfo *dev = NULL;
 
 		/* Store the device name and a reference to the NETLINK connection for
 		 * objects to use when quering for device info
 		 */
-		objdata->ethinfo->device = strdup(fetch_devs[i]);
-		objdata->ethinfo->index = -1;
-		objdata->nlc = &nlconnection;
-		objdata->nlc_users = &nlconnection_users;
 
-		/* Instantiate a new etherinfo object with the device information */
-		ethinf_py = PyCObject_FromVoidPtr(objdata, NULL);
-		if( ethinf_py ) {
-			/* Prepare the argument list for the object constructor */
-			PyObject *args = PyTuple_New(1);
-			PyTuple_SetItem(args, 0, ethinf_py);
+                dev = PyObject_New(PyEtherInfo, &PyEtherInfo_Type);
+                if( !dev ) {
+			PyErr_SetString(PyExc_OSError, strerror(errno));
+			free(fetch_devs);
+			return NULL;
+                }
+		dev->device = PyString_FromString(fetch_devs[i]);
+		dev->hwaddress = NULL;
+		dev->index = -1;
 
-			/* Create the object */
-			PyObject *dev = PyObject_CallObject((PyObject *)&ethtool_etherinfoType, args);
-			if( dev ) {
-				PyList_Append(devlist, dev);
-				Py_DECREF(dev);
-			}
-			Py_DECREF(args);
-		}
-	}
-	if( fetch_devs_len > 0 ) {
-		free(fetch_devs);
+		/* Append device object to the device list */
+		PyList_Append(devlist, (PyObject *)dev);
+		Py_DECREF(dev);
 	}
+	free(fetch_devs);
 
 	return devlist;
 }
@@ -345,7 +297,7 @@ static PyObject *get_interfaces_info(PyObject *self __unused, PyObject *args) {
 static PyObject *get_flags (PyObject *self __unused, PyObject *args)
 {
 	struct ifreq ifr;
-	char *devname;
+	const char *devname;
 	int fd, err;
 
 	if (!PyArg_ParseTuple(args, "s", &devname))
@@ -382,7 +334,7 @@ static PyObject *get_netmask (PyObject *self __unused, PyObject *args)
 {
 	struct ifreq ifr;
 	int fd, err;
-	char *devname;
+	const char *devname;
 	char netmask[20];
 
 	if (!PyArg_ParseTuple(args, "s", &devname))
@@ -426,7 +378,7 @@ static PyObject *get_broadcast(PyObject *self __unused, PyObject *args)
 {
 	struct ifreq ifr;
 	int fd, err;
-	char *devname;
+	const char *devname;
 	char broadcast[20];
 
 	if (!PyArg_ParseTuple(args, "s", &devname))
@@ -472,7 +424,7 @@ static PyObject *get_module(PyObject *self __unused, PyObject *args)
 	struct ifreq ifr;
 	int fd, err;
 	char buf[2048];
-	char *devname;
+	const char *devname;
 
 	if (!PyArg_ParseTuple(args, "s", &devname))
 		return NULL;
@@ -500,7 +452,7 @@ static PyObject *get_module(PyObject *self __unused, PyObject *args)
 		int eno = errno;
 		FILE *file;
 		int found = 0;
-		char driver[100], dev[100];
+		char driver[101], dev[101];
 		close(fd);
 
 		/* Before bailing, maybe it is a PCMCIA/PC Card? */
@@ -545,7 +497,7 @@ static PyObject *get_businfo(PyObject *self __unused, PyObject *args)
 	struct ifreq ifr;
 	int fd, err;
 	char buf[1024];
-	char *devname;
+	const char *devname;
 
 	if (!PyArg_ParseTuple(args, "s", &devname))
 		return NULL;
@@ -582,7 +534,7 @@ static PyObject *get_businfo(PyObject *self __unused, PyObject *args)
 	return PyString_FromString(((struct ethtool_drvinfo *)buf)->bus_info);
 }
 
-static int send_command(int cmd, char *devname, void *value)
+static int send_command(int cmd, const char *devname, void *value)
 {
 	/* Setup our request structure. */
 	int fd, err;
@@ -616,7 +568,7 @@ static int send_command(int cmd, char *devname, void *value)
 
 static int get_dev_value(int cmd, PyObject *args, void *value)
 {
-	char *devname;
+	const char *devname;
 	int err = -1;
 
 	if (PyArg_ParseTuple(args, "s", &devname))
@@ -639,7 +591,7 @@ static int get_dev_int_value(int cmd, PyObject *args, int *value)
 static int dev_set_int_value(int cmd, PyObject *args)
 {
 	struct ethtool_value eval;
-	char *devname;
+	const char *devname;
 
 	if (!PyArg_ParseTuple(args, "si", &devname, &eval.data))
 		return -1;
@@ -823,7 +775,7 @@ static PyObject *get_coalesce(PyObject *self __unused, PyObject *args)
 static PyObject *set_coalesce(PyObject *self __unused, PyObject *args)
 {
 	struct ethtool_coalesce coal;
-	char *devname;
+	const char *devname;
 	PyObject *dict;
 
 	if (!PyArg_ParseTuple(args, "sO", &devname, &dict))
@@ -863,7 +815,7 @@ static PyObject *get_ringparam(PyObject *self __unused, PyObject *args)
 static PyObject *set_ringparam(PyObject *self __unused, PyObject *args)
 {
 	struct ethtool_ringparam ring;
-	char *devname;
+	const char *devname;
 	PyObject *dict;
 
 	if (!PyArg_ParseTuple(args, "sO", &devname, &dict))
@@ -987,16 +939,12 @@ PyMODINIT_FUNC initethtool(void)
 	m = Py_InitModule3("ethtool", PyEthModuleMethods, "Python ethtool module");
 
 	// Prepare the ethtool.etherinfo class
-	if (PyType_Ready(&ethtool_etherinfoType) < 0)
+	if (PyType_Ready(&PyEtherInfo_Type) < 0)
 		return;
-	Py_INCREF(&ethtool_etherinfoType);
-	PyModule_AddObject(m, "etherinfo", (PyObject *)&ethtool_etherinfoType);
 
-	// Prepare the ethtool.etherinfo_ipv6addr class
-	if (PyType_Ready(&ethtool_etherinfoIPv6Type) < 0)
+	// Prepare the ethtool IPv6 and IPv4 address types
+	if (PyType_Ready(&ethtool_netlink_ip_address_Type))
 		return;
-	Py_INCREF(&ethtool_etherinfoIPv6Type);
-	PyModule_AddObject(m, "etherinfo_ipv6addr", (PyObject *)&ethtool_etherinfoIPv6Type);
 
 	// Setup constants
 	PyModule_AddIntConstant(m, "IFF_UP", IFF_UP);			/* Interface is up. */
@@ -1020,3 +968,11 @@ PyMODINIT_FUNC initethtool(void)
 	PyModule_AddStringConstant(m, "version", "python-ethtool v" VERSION);
 }
 
+
+/*
+Local variables:
+c-basic-offset: 8
+indent-tabs-mode: y
+End:
+*/
+
